# deprecation warnings are not displayed again

    Code
      wrn
    Output
      <warning/lifecycle_warning_deprecated>
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
      This warning is displayed once per session.
      Call `lifecycle::last_lifecycle_warnings()` to see where this warning was generated.

---

    Code
      wrn
    Output
      <warning/lifecycle_warning_deprecated>
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
      Call `lifecycle::last_lifecycle_warnings()` to see where this warning was generated.

---

    Code
      wrn
    Output
      <warning/lifecycle_warning_deprecated>
      Warning:
      `foo()` was deprecated in lifecycle 1.0.0.
      Call `lifecycle::last_lifecycle_warnings()` to see where this warning was generated.

