# `signal_experimental()` and `signal_superseded()` are deprecated

    Code
      signal_experimental("1.1.0", "foo()")
    Condition
      Warning:
      `signal_experimental()` was deprecated in lifecycle 1.1.0.
      i Please use `signal_stage()` instead.

---

    Code
      signal_superseded("1.1.0", "foo()")
    Condition
      Warning:
      `signal_superseded()` was deprecated in lifecycle 1.1.0.
      i Please use `signal_stage()` instead.

